﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using ThinkGeo.MapSuite.Core;
using ThinkGeo.MapSuite.DesktopEdition;
using System.Globalization;

namespace test
{
    public partial class frmPlotLatlon : Form
    {
        public frmPlotLatlon()
        {
            InitializeComponent();
        }
        GIS_Lib gis = new GIS_Lib();
        Proj4Projection proj4 = new Proj4Projection();
        private void frmPlotLatlon_Load(object sender, EventArgs e)
        {
            try
            {
                ShapeFileFeatureLayer shapefileFeatureLayer = new ShapeFileFeatureLayer(Application.StartupPath + @"\Data\Countries02.shp");
                shapefileFeatureLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle = new AreaStyle(new GeoPen(GeoColor.FromArgb(255, 128, 128, 64), 1F), new GeoSolidBrush(GeoColor.FromArgb(255, 198, 187, 151)));
                shapefileFeatureLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

                proj4.InternalProjectionParametersString = ManagedProj4Projection.GetEpsgParametersString(4326);
                proj4.ExternalProjectionParametersString = ManagedProj4Projection.GetEsriParametersString(102021);
                
                shapefileFeatureLayer.FeatureSource.Projection = proj4;

                LayerOverlay worldoverlay = new LayerOverlay();
                worldoverlay.Layers.Add("WorldLayer", shapefileFeatureLayer);
                wfMap.Overlays.Add("WorldOverlay", worldoverlay);


                wfMap.MapUnit = GeographyUnit.Meter;

                wfMap.BackgroundOverlay.BackgroundBrush = new GeoSolidBrush(GeoColor.FromArgb(255, 84, 167, 167));

                
                proj4.Open();

                double[] dist = new double[100];
                int m = 0;
                for (int k = -80; k < 90; k = k + 10)
                {
                    PointShape p1 = new PointShape(proj4.ConvertToExternalProjection(0, -90).X,proj4.ConvertToExternalProjection(0, -90).Y);
                    PointShape p2 = new PointShape(proj4.ConvertToExternalProjection(0, k).X, proj4.ConvertToExternalProjection(0, k).Y);

                    dist[m] = p1.GetDistanceTo(p2, GeographyUnit.Meter, DistanceUnit.NauticalMile);

                    m++;
                }
                PlotCircleLatLon(dist);
                wfMap.Refresh();
            }
            catch (Exception er)
            {
                MessageBox.Show(er.ToString());
            }
        }

        
           
      
      

        private void PlotCircleLatLon(double[] dist)
        {
            TGpoint tgp = new TGpoint();
            InMemoryFeatureLayer inMemoryLayer = new InMemoryFeatureLayer();
            inMemoryLayer.Open();
            inMemoryLayer.FeatureSource.BeginTransaction();
            inMemoryLayer.Columns.Add(new FeatureSourceColumn("LatLon"));
            inMemoryLayer.InternalFeatures.Clear();

            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.SimpleColors.Transparent;
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.Color = GeoColor.StandardColors.SlateGray;// GeoColor.StandardColors.Maroon;
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.DashStyle = LineDashStyle.Dot;
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.Width = 0.002F;
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.StandardColors.SlateGray, 0.002F));
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle.OuterPen.DashStyle = LineDashStyle.Dot;
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = new TextStyle("LatLon", new GeoFont("Calibri New", 8, DrawingFontStyles.Bold), new GeoSolidBrush(GeoColor.SimpleColors.White));
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.PointPlacement = PointPlacement.CenterLeft;

            inMemoryLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level07;

            LayerOverlay CircleOverLay = new LayerOverlay();
            CircleOverLay.Layers.Add("CircleLatLonLayer", inMemoryLayer);
            wfMap.Overlays.Add("CircleLatLonOverLay", CircleOverLay);
            for (int j = -179; j <= 180; j = j + 20)
            {

                for (int i = -90; i <= 90; i = i + 20)
                {
                    Feature fpoint = proj4.ConvertToExternalProjection(new Feature(BaseShape.CreateShapeFromWellKnownData("MULTIPOINT(" + j + " " + i + ")")));
                    if (i > 0 && j > 0)
                    {
                        fpoint.ColumnValues.Add("LatLon", Math.Abs(i).ToString() + "N" + Math.Abs(j).ToString() + "E");
                    }
                    if (i < 0 && j < 0)
                    {
                        fpoint.ColumnValues.Add("LatLon", Math.Abs(i).ToString() + "S" + Math.Abs(j).ToString() + "W");
                    }
                    if (i < 0 && j > 0)
                    {
                        fpoint.ColumnValues.Add("LatLon", Math.Abs(i).ToString() + "S" + Math.Abs(j).ToString() + "E");
                    }
                    if (i > 0 && j < 0)
                    {
                        fpoint.ColumnValues.Add("LatLon", Math.Abs(i).ToString() + "N" + Math.Abs(j).ToString() + "W");
                    }
                    inMemoryLayer.InternalFeatures.Add(fpoint);


                }
            }


            for (int j = -179; j <= 0; j = j + 10)
            {
                for (int i = -90; i <= 90; i = i + 10)
                {
                    Feature fline = proj4.ConvertToExternalProjection(new Feature(BaseShape.CreateShapeFromWellKnownData("LINESTRING(" + j + " " + i + "," + j + " " + (i + 10) + ")")));
                    //fline.ColumnValues.Add("LatLon", j.ToString() + " " + (i+10).ToString());
                    inMemoryLayer.InternalFeatures.Add(fline);

                }
            }
        
                Vertex vcentre = proj4.ConvertToExternalProjection(Convert.ToDouble(0), Convert.ToDouble(-90));
                PointShape pcentre = new PointShape(vcentre);
                Feature fcentre = new Feature(pcentre);
                int lat = -80;
                for (int u = 0; u < dist.Length; u++)
                {
                    if (dist[u] != 0 && dist[u] != null)
                    {

                        EllipseShape el = new EllipseShape(fcentre, dist[u], GeographyUnit.Meter, DistanceUnit.NauticalMile);
                        Feature fel = new Feature(el);
                        //fel.ColumnValues.Add("LatLon", lat.ToString());
                        lat = lat + 10;
                        inMemoryLayer.InternalFeatures.Add(fel);
                    }
                }
                inMemoryLayer.InternalFeatures.Add(fcentre);
            
            //}


            wfMap.Refresh(CircleOverLay);
        }
        private void PlotNCircle1(double[] dist)
        {
            TGpoint tgp = new TGpoint();
            InMemoryFeatureLayer inMemoryLayer = new InMemoryFeatureLayer();
            inMemoryLayer.Open();
            inMemoryLayer.FeatureSource.BeginTransaction();
            inMemoryLayer.Columns.Add(new FeatureSourceColumn("LatLon"));
            inMemoryLayer.InternalFeatures.Clear();

            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.SimpleColors.Transparent;
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.Color = GeoColor.StandardColors.SlateGray;// GeoColor.StandardColors.Maroon
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.DashStyle = LineDashStyle.Dot;
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.Width = 0.002F;
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.StandardColors.SlateGray, 0.002F));
            //inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle.OuterPen.DashStyle = LineDashStyle.Dot;
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = new TextStyle("LatLon", new GeoFont("Calibri New", 8, DrawingFontStyles.Bold), new GeoSolidBrush(GeoColor.SimpleColors.White));
            inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.PointPlacement = PointPlacement.CenterLeft;

            inMemoryLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            //inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultPointStyle.SymbolPen = new GeoPen(GeoColor.FromArgb(255, GeoColor.StandardColors.Pink), 0.5F);
            //inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.FillSolidBrush.Color = GeoColor.SimpleColors.Transparent;
            //inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.Color = GeoColor.StandardColors.Blue;// GeoColor.StandardColors.Maroon;
            //inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultAreaStyle.OutlinePen.Width = 0.002F;
            //inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultLineStyle = new LineStyle(new GeoPen(GeoColor.SimpleColors.Blue, 0.002F));
            //inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle = new TextStyle("LatLon", new GeoFont("Arial", 8), new GeoSolidBrush(GeoColor.SimpleColors.Red));
            //inMemoryLayer.ZoomLevelSet.ZoomLevel01.DefaultTextStyle.PointPlacement = PointPlacement.CenterLeft;

            //inMemoryLayer.ZoomLevelSet.ZoomLevel01.ApplyUntilZoomLevel = ApplyUntilZoomLevel.Level20;

            LayerOverlay CircleOverLay = new LayerOverlay();
            CircleOverLay.Layers.Add("CircleLayer", inMemoryLayer);
            wfMap.Overlays.Add("CircleOverLay", CircleOverLay);

            for (int j = -179; j <= 180; j = j + 10)
            {
                for (int i = -90; i <= 90; i = i + 10)
                {
                    Feature fline = proj4.ConvertToExternalProjection(new Feature(BaseShape.CreateShapeFromWellKnownData("LINESTRING(" + j + " " + i + "," + j + " " + (i + 10) + ")")));
                    //fline.ColumnValues.Add("LatLon", j.ToString() + " " + (i+10).ToString());
                    inMemoryLayer.InternalFeatures.Add(fline);
                    Feature fpoint = proj4.ConvertToExternalProjection(new Feature(BaseShape.CreateShapeFromWellKnownData("MULTIPOINT(" + j + " " + i + ")")));
                    if (i > 0 && j>0)
                    {
                        fpoint.ColumnValues.Add("LatLon",  Math.Abs(i).ToString() + "N"+ Math.Abs(j).ToString() + "E" );
                    }
                    if (i < 0 && j < 0)
                    {
                        fpoint.ColumnValues.Add("LatLon", Math.Abs(i).ToString() + "S" + Math.Abs(j).ToString() + "W");
                    }
                    if (i < 0 && j > 0)
                    {
                        fpoint.ColumnValues.Add("LatLon",Math.Abs(i).ToString() + "S" + Math.Abs(j).ToString() + "E");
                    }
                    if (i > 0 && j < 0)
                    {
                        fpoint.ColumnValues.Add("LatLon", Math.Abs(i).ToString() + "N" + Math.Abs(j).ToString() + "W");
                    }
                    inMemoryLayer.InternalFeatures.Add(fpoint);
                }
            }
            for (int j = -179; j <= 180; j = j + 10)
            {
                for (int i = -90; i <= 90; i = i + 10)
                {
                    Feature fline = proj4.ConvertToExternalProjection(new Feature(BaseShape.CreateShapeFromWellKnownData("LINESTRING(" + j + " " + i + "," + j + " " + (i + 10) + ")")));
                    //fline.ColumnValues.Add("LatLon", j.ToString() + " " + (i+10).ToString());
                    inMemoryLayer.InternalFeatures.Add(fline);

                }
            }
                Vertex vcentre = proj4.ConvertToExternalProjection(Convert.ToDouble(0), Convert.ToDouble(90));
                
                PointShape pcentre = new PointShape(vcentre);
                Feature fcentre = new Feature(pcentre);
            int lat=-80;
                for (int u = 0; u < dist.Length; u++)
                {
                    if (dist[u] != 0 && dist[u] != null)
                    {

                        EllipseShape el = new EllipseShape(fcentre, dist[u], GeographyUnit.Meter, DistanceUnit.NauticalMile);
                        Feature fel = new Feature(el);
                        //fel.ColumnValues.Add("LatLon", lat.ToString());
                        lat = lat + 10;
                        inMemoryLayer.InternalFeatures.Add(fel);
                    }
                }
                inMemoryLayer.InternalFeatures.Add(fcentre);
            //}


            wfMap.Refresh(CircleOverLay);
        }

        private void wfMap_MouseMove(object sender, MouseEventArgs e)
        {
            PointShape WorldPointR = ExtentHelper.ToWorldCoordinate(wfMap.CurrentExtent, e.X, e.Y, wfMap.Width, wfMap.Height);
            double X = e.X;
            double Y = e.Y;
            if (wfMap.MapUnit == GeographyUnit.Meter)
            {
                Vertex v = new Vertex();
                Feature fp = new Feature(WorldPointR);

                //if (proj4.ConvertToExternalProjection(wfMap.FindFeatureLayer("CountryLayer").GetBoundingBox()).Contains(fp))
                //if (WorldPointR.X < wfMap.CurrentExtent.UpperLeftPoint.X && WorldPointR.X > wfMap.CurrentExtent.LowerRightPoint.X && WorldPointR.Y < wfMap.CurrentExtent.UpperLeftPoint.Y && WorldPointR.Y > wfMap.CurrentExtent.LowerRightPoint.Y)
                // {
                v = proj4.ConvertToInternalProjection(Convert.ToDouble(WorldPointR.X), Convert.ToDouble(WorldPointR.Y));
                statusStrip1.Items[1].Text = string.Format(CultureInfo.InvariantCulture, "Long: {0:F4} Lat: {1:F4}", v.X, v.Y);
                // }
            }


        }
    }
}
